"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_native_reanimated_1 = __importDefault(require("react-native-reanimated"));
var react_1 = require("react");
var react_navigation_stack_1 = require("react-navigation-stack");
// @ts-ignore
var base_1 = require("react-native-reanimated/src/base");
var types_1 = require("../types");
exports.useCurrentValue = function (navigationState) {
    // Save navigation state
    var prevNavigationStateRef = react_1.useRef(navigationState);
    // Set up some values to track
    var currentValue = react_1.useMemo(function () { return new react_native_reanimated_1.default.Value(0); }, []);
    var durationValue = react_1.useMemo(function () { return new react_native_reanimated_1.default.Value(0); }, []);
    // Get progress from stack navigation context
    var stackCardAnimationContext = react_1.useContext(react_navigation_stack_1.StackCardAnimationContext);
    // States
    var isForward = navigationState === types_1.NavigationState.ForwardFrom ||
        navigationState === types_1.NavigationState.ForwardTo;
    var inTransition = navigationState !== types_1.NavigationState.None;
    var isFocused = navigationState === types_1.NavigationState.ForwardTo ||
        navigationState === types_1.NavigationState.BackTo;
    // console.log(
    //   name,
    //   "in transition:",
    //   inTransition,
    //   "focused:",
    //   isFocused,
    //   "forward:",
    //   isForward,
    // );
    // Remember some values
    var updateValueRef = react_1.useRef();
    // Cleanup nodes
    react_1.useEffect(function () { return function () {
        // @ts-ignore
        updateValueRef.current && updateValueRef.current.__detach();
        updateValueRef.current = undefined;
    }; }, []);
    if (prevNavigationStateRef.current !== navigationState) {
        // Save cached value
        prevNavigationStateRef.current = navigationState;
        // Detach old
        if (updateValueRef.current) {
            // @ts-ignore
            updateValueRef.current.__detach();
        }
        if (!stackCardAnimationContext) {
            return {
                current: currentValue,
                duration: durationValue,
            };
        }
        var progressValue = stackCardAnimationContext.next
            ? stackCardAnimationContext.next.progress
            : stackCardAnimationContext.current.progress;
        // Set up new
        updateValueRef.current = base_1.always(react_native_reanimated_1.default.block([
            updateCurrentProc(currentValue, splitProgressProc(normalizeProgressProc(progressValue, new react_native_reanimated_1.default.Value(isForward ? 1 : 0)), new react_native_reanimated_1.default.Value(isFocused ? 1 : 0)), new react_native_reanimated_1.default.Value(inTransition ? 1 : 0), durationValue),
        ]));
        // @ts-ignore
        updateValueRef.current.__attach();
    }
    return {
        current: currentValue,
        duration: durationValue,
    };
};
var normalizeProgressProc = react_native_reanimated_1.default.proc(function (progress, isForward) {
    return react_native_reanimated_1.default.cond(react_native_reanimated_1.default.neq(isForward, 1), react_native_reanimated_1.default.sub(1, progress), progress);
});
var splitProgressProc = react_native_reanimated_1.default.proc(function (normalizedProgress, isFocused) {
    return react_native_reanimated_1.default.cond(react_native_reanimated_1.default.eq(isFocused, 1), 
    // Focused screen
    react_native_reanimated_1.default.cond(react_native_reanimated_1.default.greaterThan(normalizedProgress, 0.5), react_native_reanimated_1.default.multiply(2, react_native_reanimated_1.default.sub(normalizedProgress, 0.5)), 0), 
    // Screen we are moving from
    react_native_reanimated_1.default.cond(react_native_reanimated_1.default.lessThan(normalizedProgress, 0.5), react_native_reanimated_1.default.multiply(normalizedProgress, 2), 1));
});
var updateCurrentProc = react_native_reanimated_1.default.proc(function (current, splitProgress, inTransition, duration) {
    return react_native_reanimated_1.default.block([
        react_native_reanimated_1.default.cond(react_native_reanimated_1.default.eq(inTransition, 1), react_native_reanimated_1.default.set(current, react_native_reanimated_1.default.divide(splitProgress, react_native_reanimated_1.default.divide(1.0, duration)))),
    ]);
});
//# sourceMappingURL=useCurrentValue.js.map